# a really simple threshold post effect

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from effect import *
from core import FBO, GLSLShader
from math import cos

class Threshold(Effect):

    def __init__(self, screenDim):
        Effect.__init__(self)

        self._bufferDim = screenDim
        self._fbo = FBO(*self._bufferDim)
        self._screenDim = screenDim
#        self.registerParameter("threshold")


        self._thresholdShader = GLSLShader(
            vertexshader = [
            """
            varying vec2 tc;
            void main(void)
            {
            vec4 v = vec4(gl_Vertex);		
            gl_Position = gl_ModelViewProjectionMatrix * v;
            gl_FrontColor = gl_Color;
            tc = gl_MultiTexCoord0.xy;
            }
            """
            ],
            fragmentshader = [
            """
            uniform sampler2D T0;
            uniform float threshold;
            uniform float frequency;
            varying vec2 tc;
            void main(void)
            {

            float fIntensity = 0.0;

            vec3 color = texture2D(T0, tc).rgb;;
               //float sine0 = cos((tc.x-tc.y) * frequency)*.5+.5;

               //float sine0 = (cos(tc.x*frequency) *.5 + .5) * (sin(tc.y*frequency) * .5 + .5);

               float d = length(vec2(tc.x-.5,tc.y-.5));
               float sine0 = cos(d*frequency);
               
               

               float intensity = (color.r + color.g + color.b) / 3.0;


               

               if (intensity*sine0 > threshold)
               fIntensity = pow( (intensity*sine0 - threshold) / threshold, 1.3);

               //if (intensity*sine0 > threshold)
               //fIntensity = 1.0;
               
               

                 
               gl_FragColor.rgb =vec3(fIntensity,fIntensity,fIntensity);
            }
            """
            ])
            
    def drawQuad(self):
        umin, umax = 0, (self._screenDim[0]) / float(self._bufferDim[0] )
        vmin, vmax = 0, (self._screenDim[1]) / float(self._bufferDim[1] )

        glBegin(GL_QUADS)
        glColor4f(1,1,1,1)

        glTexCoord2f(umin, vmin)
        glVertex2f(0,0)

        glTexCoord2f(umax, vmin)
        glVertex2f(1,0)

        glTexCoord2f(umax, vmax)
        glVertex2f(1,1)

        glTexCoord2f(umin, vmax)
        glVertex2f(0,1)

        glEnd()


    def draw(self, time, delta, start, stop):

        # settping transforms
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity();
        glOrtho(0,1,0,1,0.1,1000)

        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity();
        glTranslatef(0,0,-70)

        # disable z-buffering
        glDisable(GL_DEPTH_TEST)


        self._fbo.bindAsTexture()
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP)
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP)
        
        glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, self._screenDim[0], self._screenDim[1])

        glEnable(GL_TEXTURE_2D)
        glBlendFunc(GL_ONE,GL_ONE)
#        glEnable(GL_BLEND)
        self._thresholdShader.use()
        self._thresholdShader.uniforms.threshold = .1;
        self._thresholdShader.uniforms.frequency = 600;

        self.drawQuad()

        glDisable(GL_BLEND)

        self._thresholdShader.unUse()
        glDisable(GL_TEXTURE_2D)




